@@ -166,13 +166,12 @@ char *
 gdm_server_get_display_device (GdmServer *server)
 {
 #ifdef WITH_SYSTEMD
-        /* systemd finds the display device out on its own based on the display */
-        return NULL;
+        if (sd_booted () > 0) {
+                /* systemd finds the display device out on its own based on the display */
+                return NULL;
+        }
 #endif
 
-#ifndef WITH_CONSOLE_KIT
-        return NULL;
-#endif
         if (server->priv->display_device == NULL) {
                 server->priv->display_device =
                     _gdm_server_query_ck_for_display_device (server);
