@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -36,41 +36,37 @@
 #include <sys/prctl.h>
 #endif
 
-#include <systemd/sd-daemon.h>
-
-#ifdef ENABLE_SYSTEMD_JOURNAL
-#include <systemd/sd-journal.h>
-#endif
-
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 #include "gdm-common.h"
 
 #include "gdm-session-worker-job.h"
 
+#define GDM_SESSION_SERVER_DBUS_PATH      "/org/gnome/DisplayManager/SessionServer"
+#define GDM_SESSION_SERVER_DBUS_INTERFACE "org.gnome.DisplayManager.SessionServer"
+
 extern char **environ;
 
-struct _GdmSessionWorkerJob
-{
-        GObject         parent;
+#define GDM_SESSION_WORKER_JOB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SESSION_WORKER_JOB, GdmSessionWorkerJobPrivate))
 
+struct GdmSessionWorkerJobPrivate
+{
         char           *command;
         GPid            pid;
-        gboolean        for_reauth;
 
         guint           child_watch_id;
 
         char           *server_address;
-        char          **environment;
 };
 
 enum {
         PROP_0,
         PROP_SERVER_ADDRESS,
-        PROP_ENVIRONMENT,
-        PROP_FOR_REAUTH,
 };
 
 enum {
@@ -88,48 +84,13 @@ static void     gdm_session_worker_job_finalize (GObject         *object);
 
 G_DEFINE_TYPE (GdmSessionWorkerJob, gdm_session_worker_job, G_TYPE_OBJECT)
 
-static void
-session_worker_job_setup_journal_fds (void)
-{
-#ifdef ENABLE_SYSTEMD_JOURNAL
-        if (sd_booted () > 0) {
-                const char *identifier = "gdm-session-worker";
-                int out, err;
-
-                out = sd_journal_stream_fd (identifier, LOG_INFO, FALSE);
-                if (out < 0)
-                        return;
-
-                err = sd_journal_stream_fd (identifier, LOG_WARNING, FALSE);
-                if (err < 0) {
-                        close (out);
-                        return;
-                }
-
-                VE_IGNORE_EINTR (dup2 (out, 1));
-                VE_IGNORE_EINTR (dup2 (err, 2));
-                return;
-        }
-#endif
-        return;
-}
-
 static void
 session_worker_job_child_setup (GdmSessionWorkerJob *session_worker_job)
 {
-        sigset_t mask;
-        session_worker_job_setup_journal_fds ();
-
         /* Terminate the process when the parent dies */
 #ifdef HAVE_SYS_PRCTL_H
         prctl (PR_SET_PDEATHSIG, SIGTERM);
 #endif
-        /*
-         * Reset signal mask to default since it was altered by the
-         * manager process
-         */
-        sigemptyset (&mask);
-        sigprocmask (SIG_SETMASK, &mask, NULL);
 }
 
 static void
@@ -146,8 +107,8 @@ session_worker_job_child_watch (GPid                 pid,
                  : WIFSIGNALED (status) ? WTERMSIG (status)
                  : -1);
 
-        g_spawn_close_pid (job->pid);
-        job->pid = -1;
+        g_spawn_close_pid (job->priv->pid);
+        job->priv->pid = -1;
 
         if (WIFEXITED (status)) {
                 int code = WEXITSTATUS (status);
@@ -173,30 +134,25 @@ listify_hash (const char *key,
 }
 
 static void
-copy_environment_to_hash (GdmSessionWorkerJob *job,
-                          GHashTable          *hash)
+copy_environment_to_hash (GHashTable *hash)
 {
-        char **environment;
-        gint   i;
+        gchar **env_strv;
+        gint    i;
 
-        if (job->environment != NULL) {
-                environment = g_strdupv (job->environment);
-        } else {
-                environment = g_get_environ ();
-        }
-        for (i = 0; environment[i]; i++) {
-                char **parts;
+        env_strv = g_listenv ();
 
-                parts = g_strsplit (environment[i], "=", 2);
+        for (i = 0; env_strv [i]; i++) {
+                gchar *key = env_strv [i];
+                const gchar *value;
 
-                if (parts[0] != NULL && parts[1] != NULL) {
-                        g_hash_table_insert (hash, g_strdup (parts[0]), g_strdup (parts[1]));
-                }
+                value = g_getenv (key);
+                if (!value)
+                        continue;
 
-                g_strfreev (parts);
+                g_hash_table_insert (hash, g_strdup (key), g_strdup (value));
         }
 
-        g_strfreev (environment);
+        g_strfreev (env_strv);
 }
 
 static GPtrArray *
@@ -211,7 +167,7 @@ get_job_arguments (GdmSessionWorkerJob *job,
         args = NULL;
         argv = NULL;
         error = NULL;
-        if (! g_shell_parse_argv (job->command, NULL, &argv, &error)) {
+        if (! g_shell_parse_argv (job->priv->command, NULL, &argv, &error)) {
                 g_warning ("Could not parse command: %s", error->message);
                 g_error_free (error);
                 goto out;
@@ -240,13 +196,9 @@ get_job_environment (GdmSessionWorkerJob *job)
 
         /* create a hash table of current environment, then update keys has necessary */
         hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-        copy_environment_to_hash (job, hash);
+        copy_environment_to_hash (hash);
 
-        g_hash_table_insert (hash, g_strdup ("GDM_SESSION_DBUS_ADDRESS"), g_strdup (job->server_address));
-
-        if (job->for_reauth) {
-                g_hash_table_insert (hash, g_strdup ("GDM_SESSION_FOR_REAUTH"), g_strdup ("1"));
-        }
+        g_hash_table_insert (hash, g_strdup ("GDM_SESSION_DBUS_ADDRESS"), g_strdup (job->priv->server_address));
 
         g_hash_table_foreach (hash, (GHFunc)listify_hash, env);
         g_hash_table_destroy (hash);
@@ -268,7 +220,7 @@ gdm_session_worker_job_spawn (GdmSessionWorkerJob *session_worker_job,
         ret = FALSE;
 
         g_debug ("GdmSessionWorkerJob: Running session_worker_job process: %s %s",
-                 name != NULL? name : "", session_worker_job->command);
+                 name != NULL? name : "", session_worker_job->priv->command);
 
         args = get_job_arguments (session_worker_job, name);
 
@@ -284,7 +236,7 @@ gdm_session_worker_job_spawn (GdmSessionWorkerJob *session_worker_job,
                                         G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_FILE_AND_ARGV_ZERO,
                                         (GSpawnChildSetupFunc)session_worker_job_child_setup,
                                         session_worker_job,
-                                        &session_worker_job->pid,
+                                        &session_worker_job->priv->pid,
                                         NULL,
                                         NULL,
                                         NULL,
@@ -298,16 +250,16 @@ gdm_session_worker_job_spawn (GdmSessionWorkerJob *session_worker_job,
 
         if (! ret) {
                 g_warning ("Could not start command '%s': %s",
-                           session_worker_job->command,
+                           session_worker_job->priv->command,
                            error->message);
                 g_error_free (error);
         } else {
-                g_debug ("GdmSessionWorkerJob: : SessionWorkerJob on pid %d", (int)session_worker_job->pid);
+                g_debug ("GdmSessionWorkerJob: : SessionWorkerJob on pid %d", (int)session_worker_job->priv->pid);
         }
 
-        session_worker_job->child_watch_id = g_child_watch_add (session_worker_job->pid,
-                                                                (GChildWatchFunc)session_worker_job_child_watch,
-                                                                session_worker_job);
+        session_worker_job->priv->child_watch_id = g_child_watch_add (session_worker_job->priv->pid,
+                                                                      (GChildWatchFunc)session_worker_job_child_watch,
+                                                                      session_worker_job);
 
         return ret;
 }
@@ -336,8 +288,8 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
 {
         int exit_status;
 
-        g_debug ("GdmSessionWorkerJob: Waiting on process %d", session_worker_job->pid);
-        exit_status = gdm_wait_on_and_disown_pid (session_worker_job->pid, 5);
+        g_debug ("GdmSessionWorkerJob: Waiting on process %d", session_worker_job->priv->pid);
+        exit_status = gdm_wait_on_and_disown_pid (session_worker_job->priv->pid, 5);
 
         if (WIFEXITED (exit_status) && (WEXITSTATUS (exit_status) != 0)) {
                 g_debug ("GdmSessionWorkerJob: Wait on child process failed");
@@ -345,8 +297,8 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
                 /* exited normally */
         }
 
-        g_spawn_close_pid (session_worker_job->pid);
-        session_worker_job->pid = -1;
+        g_spawn_close_pid (session_worker_job->priv->pid);
+        session_worker_job->priv->pid = -1;
 
         g_debug ("GdmSessionWorkerJob: SessionWorkerJob died");
 }
@@ -354,14 +306,14 @@ handle_session_worker_job_death (GdmSessionWorkerJob *session_worker_job)
 void
 gdm_session_worker_job_stop_now (GdmSessionWorkerJob *session_worker_job)
 {
-        if (session_worker_job->pid <= 1) {
+        if (session_worker_job->priv->pid <= 1) {
                 return;
         }
 
         /* remove watch source before we can wait on child */
-        if (session_worker_job->child_watch_id > 0) {
-                g_source_remove (session_worker_job->child_watch_id);
-                session_worker_job->child_watch_id = 0;
+        if (session_worker_job->priv->child_watch_id > 0) {
+                g_source_remove (session_worker_job->priv->child_watch_id);
+                session_worker_job->priv->child_watch_id = 0;
         }
 
         gdm_session_worker_job_stop (session_worker_job);
@@ -373,13 +325,13 @@ gdm_session_worker_job_stop (GdmSessionWorkerJob *session_worker_job)
 {
         int res;
 
-        if (session_worker_job->pid <= 1) {
+        if (session_worker_job->priv->pid <= 1) {
                 return;
         }
 
-        g_debug ("GdmSessionWorkerJob: Stopping job pid:%d", session_worker_job->pid);
+        g_debug ("GdmSessionWorkerJob: Stopping job pid:%d", session_worker_job->priv->pid);
 
-        res = gdm_signal_pid (session_worker_job->pid, SIGTERM);
+        res = gdm_signal_pid (session_worker_job->priv->pid, SIGTERM);
 
         if (res < 0) {
                 g_warning ("Unable to kill session worker process");
@@ -390,7 +342,7 @@ GPid
 gdm_session_worker_job_get_pid (GdmSessionWorkerJob *session_worker_job)
 {
         g_return_val_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job), 0);
-        return session_worker_job->pid;
+        return session_worker_job->priv->pid;
 }
 
 void
@@ -399,26 +351,8 @@ gdm_session_worker_job_set_server_address (GdmSessionWorkerJob *session_worker_j
 {
         g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
 
-        g_free (session_worker_job->server_address);
-        session_worker_job->server_address = g_strdup (address);
-}
-
-void
-gdm_session_worker_job_set_for_reauth (GdmSessionWorkerJob *session_worker_job,
-                                       gboolean             for_reauth)
-{
-        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
-
-        session_worker_job->for_reauth = for_reauth;
-}
-
-void
-gdm_session_worker_job_set_environment (GdmSessionWorkerJob *session_worker_job,
-                                        const char * const  *environment)
-{
-        g_return_if_fail (GDM_IS_SESSION_WORKER_JOB (session_worker_job));
-
-        session_worker_job->environment = g_strdupv ((char **) environment);
+        g_free (session_worker_job->priv->server_address);
+        session_worker_job->priv->server_address = g_strdup (address);
 }
 
 static void
@@ -435,12 +369,6 @@ gdm_session_worker_job_set_property (GObject      *object,
         case PROP_SERVER_ADDRESS:
                 gdm_session_worker_job_set_server_address (self, g_value_get_string (value));
                 break;
-        case PROP_FOR_REAUTH:
-                gdm_session_worker_job_set_for_reauth (self, g_value_get_boolean (value));
-                break;
-        case PROP_ENVIRONMENT:
-                gdm_session_worker_job_set_environment (self, g_value_get_pointer (value));
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -459,13 +387,7 @@ gdm_session_worker_job_get_property (GObject    *object,
 
         switch (prop_id) {
         case PROP_SERVER_ADDRESS:
-                g_value_set_string (value, self->server_address);
-                break;
-        case PROP_FOR_REAUTH:
-                g_value_set_boolean (value, self->for_reauth);
-                break;
-        case PROP_ENVIRONMENT:
-                g_value_set_pointer (value, self->environment);
+                g_value_set_string (value, self->priv->server_address);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -497,31 +419,20 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
         object_class->constructor = gdm_session_worker_job_constructor;
         object_class->finalize = gdm_session_worker_job_finalize;
 
+        g_type_class_add_private (klass, sizeof (GdmSessionWorkerJobPrivate));
+
         g_object_class_install_property (object_class,
                                          PROP_SERVER_ADDRESS,
                                          g_param_spec_string ("server-address",
                                                               "server address",
                                                               "server address",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
-        g_object_class_install_property (object_class,
-                                         PROP_FOR_REAUTH,
-                                         g_param_spec_boolean ("for-reauth",
-                                                               "for reauth",
-                                                               "for reauth",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
-        g_object_class_install_property (object_class,
-                                         PROP_ENVIRONMENT,
-                                         g_param_spec_pointer ("environment",
-                                                               "environment",
-                                                               "environment",
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         signals [STARTED] =
                 g_signal_new ("started",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              0,
+                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, started),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__VOID,
@@ -531,7 +442,7 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
                 g_signal_new ("exited",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              0,
+                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, exited),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__INT,
@@ -543,7 +454,7 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
                 g_signal_new ("died",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              0,
+                              G_STRUCT_OFFSET (GdmSessionWorkerJobClass, died),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__INT,
@@ -555,9 +466,12 @@ gdm_session_worker_job_class_init (GdmSessionWorkerJobClass *klass)
 static void
 gdm_session_worker_job_init (GdmSessionWorkerJob *session_worker_job)
 {
-        session_worker_job->pid = -1;
 
-        session_worker_job->command = g_strdup (LIBEXECDIR "/gdm-session-worker");
+        session_worker_job->priv = GDM_SESSION_WORKER_JOB_GET_PRIVATE (session_worker_job);
+
+        session_worker_job->priv->pid = -1;
+
+        session_worker_job->priv->command = g_strdup (LIBEXECDIR "/gdm-session-worker");
 }
 
 static void
@@ -570,10 +484,12 @@ gdm_session_worker_job_finalize (GObject *object)
 
         session_worker_job = GDM_SESSION_WORKER_JOB (object);
 
+        g_return_if_fail (session_worker_job->priv != NULL);
+
         gdm_session_worker_job_stop (session_worker_job);
 
-        g_free (session_worker_job->command);
-        g_free (session_worker_job->server_address);
+        g_free (session_worker_job->priv->command);
+        g_free (session_worker_job->priv->server_address);
 
         G_OBJECT_CLASS (gdm_session_worker_job_parent_class)->finalize (object);
 }
